from tkinter import *
from tkinter import filedialog
import ADS1256
import paho.mqtt.client as mqtt
from datetime import datetime
from numpy import array
import socket
import csv

#actual time stamp
now=datetime.now()
time_stamp=now.strftime("%d.%m.%y  %H:%M:%S")
print("Hall-pi  started @ " + time_stamp )
print("Version 1.0 (c) Peter Bluemler, University of Mainz, Germany 4.2020")
print(" ")

#try to get the IP address to establish a MQTT client
hostname = socket.gethostname()
sock=socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
try: 
    sock.connect(("8.8.8.8",80))
    IPAddr=sock.getsockname()[0]
    print(hostname + " connected via IP-address: "+ IPAddr )
except socket.error:
    print("WARNING: No internet connection ")
    IPAddr=""
if IPAddr !="":  #if there is an IP-address establish MQTT connection
    client = mqtt.Client()
    client.connect(IPAddr,1883,60)
    client.publish("hallpi","Connection established! @ "+time_stamp)

#the global variables used in the program
global B_calib, B_offset, average, digits, unit_mult, comp

#read default values from last leave-procedure from file
filename='defaults.csv'
try:
   with open(filename,'r') as file:
       reader=csv.reader(file)
       data=next(reader)
       B_calib=array([float(data[0]),float(data[1]),float(data[2])])
       B_offset=array([float(data[3]),float(data[4]),float(data[5])])
       average=int(data[6])
       digits=int(data[7])
       unit_mult=float(data[8])
       comp=[int(data[9]),int(data[10]),int(data[11])]
except IOError:
    print("WARNING: could not fine default.csv guessing values")
    B_calib=array([2.0,2.0,2.0])
    B_offset=array([1.1,1.1,1.1])
    average=10
    digits=1
    unit_mult=1
    comp=[1,1,1]    

#set the ADC 
ADC = ADS1256.ADS1256() # define the ADC 
ADC.ADS1256_init()
print("_____________________________________________________")



###############  FUNCTIONS ####################

def read_ADC(): 
   ADC_volts = array(ADC.ADS1256_GetAll())      #read the voltages from the ADC
   ADC_volts = ADC_volts*5.0/0x7fffff           # multiply all with 5.0 Volts and divide by the bits 2^23 
   return ADC_volts


def fetch(entries): # this function handles values entered into the various entry fields
   # 3-5 = calib values, 6-8 offsets, 9=averages, 10=digits
   global B_calib, B_offsets, average, digits
   for k in range(0,len(entries)):
      entry=entries[k]
      if k == 3:
          B_calib[0] =float(entry.get())
          entry.delete(0,END)
          entry.insert(0,B_calib[0])
      if k == 4:
          B_calib[1] =float(entry.get())
          entry.delete(0,END)
          entry.insert(0,B_calib[1])
      if k == 5:
          B_calib[2] =float(entry.get())
          entry.delete(0,END)
          entry.insert(0,B_calib[2])
      if k == 6:
          B_offset[0] =float(entry.get())
          entry.delete(0,END)
          entry.insert(0,B_offset[0])
      if k == 7:
          B_offset[1] =float(entry.get())
          entry.delete(0,END)
          entry.insert(0,B_offset[1])
      if k == 8:
          B_offset[2] =float(entry.get())
          entry.delete(0,END)
          entry.insert(0,B_offset[2])
      if k ==9:
         average=int(entry.get())
         if average < 1:
            average = 1
         entry.delete(0,END)
         entry.insert(0,average)
      if k == 10:
         digits=int(entry.get())
         if digits<0:
            digits=0
         elif digits >8:
            digits=8
         entry.delete(0,END)
         entry.insert(0,digits)
             
   
def setzero(entries): #determines the average of 100 measurements at zero field to calc. offsets 
    global B_offset
    ave_zero=100      # number of averages
    V_avg=array([0,0,0])
    for k in range(0,ave_zero): #sample flux
        volts=read_ADC()
        V_avg=V_avg+volts[5:8]
    B_offset=V_avg/ave_zero     #calc average
    for k in range(0,len(entries)): #set the entries to the new value
        entry=entries[k]
        if k == 6:
            entry.delete(0,END)
            entry.insert(0,B_offset[0])
        if k == 7:
            entry.delete(0,END)
            entry.insert(0,B_offset[1])
        if k == 8:
            entry.delete(0,END)
            entry.insert(0,B_offset[2])
    
def set_unit(v):  #set the unit multiplier from the 
    global unit_mult
    multiplier={1:1, 2:1000, 3:10000, 4: 10}
    unit_mult=multiplier[v]

def set_comp(c): #just to assign the comp to a global variable
    global comp
    comp=c

def display_flux(entries):
   #read ADC and display calibrated Bxyz and T values
   def check_ADC(): 
      global B_calib, B_offset, average, digits, unit_mult,comp
      V_avg=array([0,0,0])
      for j in range(0,average):    #do the averaging
           volts=read_ADC()
           V_avg = V_avg + volts[5:8]
      B_act=V_avg/average
      Bstr=["Bx","By","Bz"]   #define output string
      Bamp=0
      for k in range(0,3):
          entry=entries[k]
          if comp[k]==1:  # if checkbutton is ON 
              pubstr=Bstr[k] + " = "+"% "+str(digits+7)+"."+str(digits)+"f"  #compose output string
              pubstr=pubstr% round((B_act[k]-B_offset[k])*B_calib[k]*unit_mult,digits)
              Bamp=Bamp+((B_act[k]-B_offset[k])*B_calib[k]*unit_mult)**2
              if unit_mult==1:
                  pubstr=pubstr + " T" 
              if unit_mult==1000:
                  pubstr=pubstr + " mT" 
              if unit_mult==10000:
                  pubstr=pubstr + " G" 
              if unit_mult==10:
                  pubstr=pubstr + " kG" 
              entry.config(text=pubstr)
              if IPAddr !="":      # if connected via MQTT write string to client
                 client.publish("hallpi",pubstr)
          else:
              pubstr=Bstr[k] + " = ---- " #if checkbutton is not ticked display ----
              entry.config(text=pubstr)
      #calc and display B amplitude         
      entry=entries[11]
      Bamp=Bamp**0.5    
      pubstr="|B| = "+"% "+str(digits+7)+"."+str(digits)+"f"
      pubstr=pubstr% round(Bamp,digits)
      if unit_mult==1:
         pubstr=pubstr + " T" 
      if unit_mult==1000:
         pubstr=pubstr + " mT" 
      if unit_mult==10000:
         pubstr=pubstr + " G" 
      if unit_mult==10:
         pubstr=pubstr + " kG" 
      entry.config(text=pubstr)
      entry.after(1,check_ADC)
   check_ADC()

def load_probe(window):  #read in the data writen with save_probe
    global B_calib, B_offset, average, digits, unit_mult, comp
    filename=filedialog.askopenfilename(title="Save settings as", filetypes=(("csv files","*.csv"),("all files","*.*")))
    try:
       with open(filename,'r') as file:
           reader=csv.reader(file)
           data=next(reader)
           B_calib=array([float(data[0]),float(data[1]),float(data[2])])
           B_offset=array([float(data[3]),float(data[4]),float(data[5])])
           average=int(data[6])
           digits=int(data[7])
           unit_mult=float(data[8])
           comp=[int(data[9]),int(data[10]),int(data[11])]
    except IOError:
        print("ERROR reading "+filename+"   values not changed!")
    
    
def save_probe(window):
    global B_calib, B_offset, average, digits, unit_mult, comp
    filename=filedialog.asksaveasfilename(title="Save settings as", filetypes=(("csv files","*.csv"),("all files","*.*")))
    file=open(filename,'w')  #save settings to default
    with file:
       writer=csv.writer(file)
       data=list(B_calib)+list(B_offset)+[average,digits,unit_mult]+comp
       print("saved the following data to "+filename)
       print(" ")
       writer.writerow(data)


def leave(entries):
   global B_calib, B_offset, average, digits, unit_mult, comp
   file=open('defaults.csv','w')  #save settings to default
   with file:
       writer=csv.writer(file)
       data=list(B_calib)+list(B_offset)+[average,digits,unit_mult]+comp
       print("saved the following data to default.csv")
       print(" ")
       writer.writerow(data)
   quit()


def makeform(window):
   #MAIN DISPLAY OF FIELD
   global B_calib, B_offset, average, digits, unit_mult, comp
   B_act=read_ADC()
   entries = []
   # draw some boxes
   Mc1=Canvas(window,width=800,height=440)
   Mc1.place(x=1,y=1)
   Mc1.create_rectangle(5,40,795,110)
   Mc1.create_rectangle(5,40,795,180)
   Mc1.create_rectangle(5,40,795,250)
   lab_dummy=Label(window,text='Magnetic flux monitor', fg='magenta4', font=("Arial 16 bold")).place(x=290,y=8)
   lab_dummy=Label(window,text='© 2020 Peter Blümler', font=("Arial 8")).place(x=600,y=420)
   #  the flux labels Bx,y,z
   lab_Bx=Label(window,text='Bx = '+str(round((B_act[5]-B_offset[0])*B_calib[0],digits))+' T', fg='blue', font=("Arial",24),justify=LEFT)
   lab_Bx.place(x=50,y=55)
   lab_By=Label(window,text='By = '+str(round((B_act[6]-B_offset[1])*B_calib[1],digits))+' T', fg='blue', font=("Arial",24),justify=LEFT)
   lab_By.place(x=50,y=125)
   lab_Bz=Label(window,text='Bz = '+str(round((B_act[7]-B_offset[2])*B_calib[2],digits))+' T', fg='blue', font=("Arial",24),justify=LEFT)
   lab_Bz.place(x=50,y=195)
   lab_Ba=Label(window,text='|B| = '+str(round((B_act[7]-B_offset[2])*B_calib[2],digits))+' T',  font=("Arial",24),justify=LEFT)
   lab_Ba.place(x=50,y=265)
   #calibration values of ADC -> magnetic field 
   lab_Bcx=Label(window, width=16, text='calib. value [T/V]:', justify=RIGHT).place(x=565,y=51)
   ent_Bcx=Entry(window, width=12)
   ent_Bcx.place(x=690,y=50)
   ent_Bcx.insert(0, B_calib[0])
   lab_Bcy=Label(window, width=16, text='calib. value [T/V]:', justify=RIGHT).place(x=565,y=121)
   ent_Bcy=Entry(window, width=12)
   ent_Bcy.place(x=690,y=120)
   ent_Bcy.insert(0, B_calib[1])
   lab_Bcz=Label(window, width=16, text='calib. value [T/V]:', justify=RIGHT).place(x=565,y=191)
   ent_Bcz=Entry(window, width=12)
   ent_Bcz.place(x=690,y=190)
   ent_Bcz.insert(0, B_calib[2])
   #offset values of ADC -> magnetic field 
   lab_Box=Label(window, width=16, text='offset [V]:', justify=RIGHT).place(x=540,y=81)
   ent_Box=Entry(window, width=18)
   ent_Box.place(x=642,y=80)
   ent_Box.insert(0, B_offset[0])
   lab_Boy=Label(window, width=16, text='offset [V]:', justify=RIGHT).place(x=540,y=151)
   ent_Boy=Entry(window, width=18)
   ent_Boy.place(x=642,y=150)
   ent_Boy.insert(0, B_offset[1])
   lab_Boz=Label(window, width=16, text='offset [V]:', justify=RIGHT).place(x=540,y=221)
   ent_Boz=Entry(window, width=18)
   ent_Boz.place(x=642,y=220)
   ent_Boz.insert(0, B_offset[2])
   #average entry
   lab_avg=Label(window, width=10, text='averages:', justify=RIGHT)
   lab_avg.place(x=300,y=330)
   ent_avg=Entry(window, width=6)
   ent_avg.place(x=380,y=330)
   ent_avg.insert(0, average)
   #decimal places
   lab_dig=Label(window, width=14, text='decimal places:', justify=RIGHT)
   lab_dig.place(x=265,y=355)
   ent_dig=Entry(window, width=6)
   ent_dig.place(x=380,y=355)
   ent_dig.insert(0, digits)
   #set zero button
   zero_but=Button(window, text='set Zero', fg='blue', command=lambda: setzero(entries))
   zero_but.place(x=710,y=260)
   #set load_probe button
   load_but=Button(window, text='load probe',  command=lambda: load_probe(window))
   load_but.place(x=100,y=405)
   #set save_zero button
   save_but=Button(window, text='save probe',  command=lambda: save_probe(window))
   save_but.place(x=230,y=405)
   #quit button
   quit_but=Button(window, text='QUIT', fg='red', command=lambda: leave(entries))
   quit_but.place(x=732,y=405)
   #
   #Finally the interacting elements of the GUI are defined in an entry-list
   entries.append(lab_Bx)     # [0] = Bx main display label
   entries.append(lab_By)     # [1] = By main display label
   entries.append(lab_Bz)     # [2] = Bz main display label
   entries.append(ent_Bcx)    # [3] = Entry field of Bx calibration 
   entries.append(ent_Bcy)    # [4] = Entry field of By calibration 
   entries.append(ent_Bcz)    # [5] = Entry field of Bz calibration 
   entries.append(ent_Box)    # [6] = Entry field of Bx offset 
   entries.append(ent_Boy)    # [7] = Entry field of By offset 
   entries.append(ent_Boz)    # [8] = Entry field of Bz offset 
   entries.append(ent_avg)    # [9] = average label
   entries.append(ent_dig)    # [10] = digits label
   entries.append(lab_Ba)     # [11] = amplitude of field
   return entries


# The main loop
if __name__ == '__main__':
   window = Tk()
   v=IntVar()
   # define unit multiplactor for radiobuttons
   if unit_mult==1:
      v.set(1)
   if unit_mult==1000:
      v.set(2)
   if unit_mult==10000:
      v.set(3)
   if unit_mult==10:
      v.set(4)
   units=[("Tesla [T]",1),("milliTesla [mT]",2),("Gauss [G]",3),("kiloGauss [kG]",4)]
   #make window
   window.title('Hall-pi flux monitor')
   window.geometry("800x440+0+0") #adapted to the 5" screen (800 * 440 pixels) 0+0 is top left corner
   ents = makeform(window)        #make the GUI layout and get the entries
   # make the radiobutton field for the units
   rad_lab=Label(window,text="units:", font=('Arial 12 bold')).place(x=525,y=305)
   for txt, val in units:
       rad_but=Radiobutton(window,text=txt, variable=v, command=lambda: set_unit(v.get()),value=val)
       rad_but.place(x=500,y=310+val*20)
   # and checkbuttons for the flux components (could not find a more elegant way)
   csx=IntVar()
   csx.set(comp[0])
   csy=IntVar()
   csy.set(comp[1])
   csz=IntVar()
   csz.set(comp[2])
   chBBx=Checkbutton(window, variable=csx, command=lambda: set_comp([csx.get(),csy.get(),csz.get()])).place(x=15,y=63)
   chBBy=Checkbutton(window, variable=csy, command=lambda: set_comp([csx.get(),csy.get(),csz.get()])).place(x=15,y=133)
   chBBz=Checkbutton(window, variable=csz, command=lambda: set_comp([csx.get(),csy.get(),csz.get()])).place(x=15,y=203)
   comp=[csx.get(),csy.get(),csz.get()] 
   #
   # in case return is pressed after changing entries they are passed to function fetch
   window.bind('<Return>', (lambda event, e=ents: fetch(e)))
   display_flux(ents)  # measure and display flux 
   window.mainloop()   # loop
   