from tkinter import *
import ADS1256
import paho.mqtt.client as mqtt
from datetime import datetime
import socket
import csv

#actual time stamp
now=datetime.now()
time_stamp=now.strftime("%d.%m.%y  %H:%M:%S")
print("Hall-pi  started @ " + time_stamp )
print("Version 1.0 (c) Peter Bluemler, University of Mainz, Germany 4.2020")
print(" ")
#try to get the IP address to establish a MQTT client
hostname = socket.gethostname()
sock=socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
try: 
    sock.connect(("8.8.8.8",80))
    IPAddr=sock.getsockname()[0]
    print(hostname + " connected via IP-address: "+ IPAddr )

except socket.error:
    print("WARNING: No internet connection ")
    IPAddr=""

if IPAddr !="":
    client = mqtt.Client()
    client.connect(IPAddr,1883,60)
    client.publish("hallpi","Connection established! @ "+time_stamp)

global B_offset, B_calib, B_act, digits, average, unit_mult
#read default values from last leave-procedure from file
file=open('defaults.csv','r')
with file:
    reader=csv.reader(file)
    data=next(reader)
#B_offset=float(data[0])
#B_calib=float(data[1])
#digits = int(data[2])
#average = int(data[3])
#unit_mult = float(data[4])
    
B_offset=2
B_calib=1.2
digits=2
average=10
unit_mult=1
    


ADC = ADS1256.ADS1256() # define the ADC 
ADC.ADS1256_init()
print("_____________________________________________________")
###############  FUNCTIONS ####################

def read_ADC():
   ADC_volts = ADC.ADS1256_GetAll()      #read the voltages from the ADC
   ADC_volts =[k*5.0 for k in ADC_volts] # multiply all with 5.0 Volts
   return ADC_volts



def fetch(entries):
   global average, digits, B_calib
   for k in range(0,len(entries)):
      entry=entries[k]
      if k == 1:
         average=int(entry.get())
         if average < 1:
            average = 1
         entry.delete(0,END)
         entry.insert(0,average)
      if k == 2:
         digits=int(entry.get())
         if digits<0:
            digits=0
         elif digits >8:
            digits=8
         entry.delete(0,END)
         entry.insert(0,digits)
      if k == 3:
          B_calib =float(entry.get())
          entry.delete(0,END)
          entry.insert(0,B_calib)
             
   
def setzero(entries):
    global B_offset
    
    ave_zero=100
    B_av=0
    for k in range(1,ave_zero+1):
        B_act=read_ADC()
        B_av=B_av+B_act
    B_offset=B_av/ave_zero
    
def set_unit(v):
    global unit_mult
    multiplier={1:1, 2:1000, 3:10000, 4: 10}
    unit_mult=multiplier[v]

def display_flux(entries):
   #read and display B and T values
   def check_ADC():
      B_av=0
      for k in range(1,average+1):
           B_act=read_ADC()
           B_av=B_av+B_act[7]
      B_av=B_av/average  
      entry=entries[0]
      strtxt="B = "+"% "+str(digits+7)+"."+str(digits)+"f"
      if unit_mult==1:
         entry.config(text=strtxt% round((B_av-B_offset)*B_calib,digits)+" T")
      if unit_mult==1000:
         entry.config(text=strtxt% round((B_av-B_offset)*B_calib*unit_mult,digits)+" mT")
      if unit_mult==10000:
         entry.config(text=strtxt% round((B_av-B_offset)*B_calib*unit_mult,digits)+" G")
      if unit_mult==10:
         entry.config(text=strtxt% round((B_av-B_offset)*B_calib*unit_mult,digits)+" kG")
         
      entry.after(1,check_ADC)
   check_ADC()

def leave(entries):
   global B_offset, B_calib, B_act, digits, average, unit_mult
   file=open('defaults.csv','w')
   with file:
       writer=csv.writer(file)
       writer.writerow([B_offset, B_calib, digits, average, unit_mult])
   quit()


def makeform(root):
   #MAIN DISPLAY OF FIELD
   B_act=0
   B_act=read_ADC()
 
   entries = []
   Mc1=Canvas(window,width=500,height=100)
   Mc1.place(x=1,y=1)
   Mc1.create_rectangle(2,2,450,60)
   lab_Bx=Label(window,text='B = '+str(round((B_act[7]-B_offset)*B_calib,digits))+' T', fg='blue', font=("Arial",24),justify=LEFT)
   lab_Bx.place(x=20,y=10)
   #average entry
   lab_avg=Label(window, width=10, text='averages:', justify=LEFT)
   lab_avg.place(x=10,y=80)
   ent_avg=Entry(window, width=6)
   ent_avg.place(x=100,y=80)
   ent_avg.insert(0, average)
   #decimal places
   lab_dig=Label(window, width=10, text='dec. place:', justify=LEFT)
   lab_dig.place(x=10,y=100)
   ent_dig=Entry(window, width=6)
   ent_dig.place(x=100,y=100)
   ent_dig.insert(0, digits)
   #calibration value of ADC -> magnetic field
   lab_cal=Label(window, width=10, text='calib. value:', justify=LEFT)
   lab_cal.place(x=10,y=150)
   ent_cal=Entry(window, width=6)
   ent_cal.place(x=100,y=150)
   ent_cal.insert(0, B_calib)
   #set zero button
   zero_but=Button(window, text='set Zero', fg='blue', command=lambda: setzero(entries))
   zero_but.place(x=180,y=120)
   #quiT button
   quit_but=Button(window, text='QUIT', fg='red', command=lambda: leave(entries))
   quit_but.place(x=300,y=170)
   #copyright
   lab_copyright=Label(window,text='© 2020 Peter Blümler', font=("Arial",8))
   lab_copyright.place(x=150,y=185)

   entries.append(lab_Bx)     # [0] = B main display label
   entries.append(ent_avg)    # [1] = average label
   entries.append(ent_dig)    # [2] = digits label
   entries.append(ent_cal)    # [3] = calibration value
   
 
 
   return entries

if __name__ == '__main__':
   window = Tk()
   v=IntVar()
   if unit_mult==1:
      v.set(1)
   if unit_mult==1000:
      v.set(2)
   if unit_mult==10000:
      v.set(3)
   if unit_mult==10:
      v.set(4)
   units=[("Tesla [T]",1),("milliTesla [mT]",2),("Gauss [G]",3),("kiloGauss [kG]",4)]
   window.title('Hall-pi flux monitor')
   window.geometry("800x440+0+0")
   ents = makeform(window)
   rad_lab=Label(window,text="units:", font=('bold',10))
   rad_lab.place(x=230,y=80)
   for txt, val in units:
       rad_but=Radiobutton(window,text=txt, variable=v, command=lambda: set_unit(v.get()),value=val)
       rad_but.place(x=280,y=60+val*20)
   window.bind('<Return>', (lambda event, e=ents: fetch(e)))
   display_flux(ents)
   window.mainloop()
   

